package com.ether;

import com.ether.annotation.ControllerMethod;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public class MethodDescription
{
    MethodDescription( @NotNull final Method method, @NotNull final ControllerMethod annotation )
    {
        _method = method;
        _view = annotation.view();
        _defaultView = annotation.defaultView();
        _isJsonResult = annotation.jsonResult();
        _isLoginMandatory = annotation.loginMandatory();
        _isRequestMandatory = annotation.requestMandatory();
        _isBackofficeMethod = annotation.backofficeMethod();
        _params = new ArrayList<ParamDescription>();
    }

    public void addNullParam()
    {
        _params.add( null );
    }

    public void addParam( @NotNull final String paramName, @NotNull final Class<?> paramType, final boolean mandatory, final boolean usingJSON )
    {
        _params.add( new ParamDescription( paramName, paramType, mandatory, usingJSON ) );
    }

    @NotNull
    public String getView()
    {
        return _view;
    }

    @Nullable
    public String getDefaultView()
    {
        return _defaultView;
    }

    public void setDefaultView( @Nullable final String defaultView )
    {
        _defaultView = defaultView;
    }

    @NotNull
    public Method getMethod()
    {
        return _method;
    }

    public Boolean isJsonResult()
    {
        return _isJsonResult;
    }

    @Nullable
    public List<ParamDescription> getParams()
    {
        return _params;
    }

    public String getName()
    {
        return _method.getName();
    }

    @Override
    public String toString()
    {
        return _method.toString();
    }

    public Boolean isLoginMandatory()
    {
        return _isLoginMandatory;
    }

    public void setLoginMandatory( final Boolean loginMandatory )
    {
        _isLoginMandatory = loginMandatory;
    }

    public Boolean isRequestMandatory()
    {
        return _isRequestMandatory;
    }

    public void setRequestMandatory( final Boolean requestMandatory )
    {
        _isRequestMandatory = requestMandatory;
    }

    public Boolean isBackofficeMethod()
    {
        return _isBackofficeMethod;
    }

    public void setBackofficeMethod( final Boolean backofficeMethod )
    {
        _isBackofficeMethod = backofficeMethod;
    }

    @NotNull
    private Method _method;
    @NotNull
    private String _view;
    @Nullable
    private String _defaultView;
    private Boolean _isJsonResult;
    @Nullable
    private List<ParamDescription> _params;
    private Boolean _isLoginMandatory;
    private Boolean _isRequestMandatory;
    private Boolean _isBackofficeMethod;
}
