package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.ParamName;
import com.ether.user.User;
import com.ether.user.UserRole;
import com.ether.user.UserState;
import com.medias.mail.Mail;
import com.medias.mail.MailFactory;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.mail.MessagingException;
import java.security.DigestException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author vmipsl
 * @date 13 nov 2011
 */
public class ControllerBackoffice
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(view = VIEW_BO_INDEX, backofficeMethod = true, defaultView = VIEW_BO_INDEX)
    public Map<String, Object> home()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_BO_USER, backofficeMethod = true, defaultView = VIEW_BO_INDEX)
    public Map<String, Object> viewUser()
            throws ServiceException
    {
        final List<User> waitingUsers = getEtherService().getUsersByState( UserState.WAITING );
        final List<User> users = getEtherService().getAllUsersByNameOrder();

        final Map<String, Object> model = new HashMap<String, Object>();
        model.put( "jSonWaitingUsers", getJsonHelper().toJSON( waitingUsers ) );
        model.put( "jSonUsers", getJsonHelper().toJSON( users ) );
        model.put( "jSonUserStates", getJSONUserStates() );
        model.put( "jSonUserRoles", getJSONUserRoles() );
        return model;
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(jsonResult = true)
    public JSONObject addUser( @NotNull @ParamName(ParameterConstants.PARAMETER_NAME) final String lastName,
                               @Nullable @ParamName(ParameterConstants.PARAMETER_FIRST_NAME) final String firstName,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_EMAIL) final String email,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_PWD) final String pwd,
                               @Nullable @ParamName(ParameterConstants.PARAMETER_ROLE) final String role,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_STATE) final String state,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_HAS_ACCESS) final Boolean hasAccessToBO )
            throws WebException, ServiceException
    {
        try
        {
            final User existingUser = getEtherService().getUserByEmail( email );
            if( null == existingUser )
            {
                final Date creationDate = new Date();
                final String encryptedPassword = EtherHelper.encryptPassword( pwd );
                final User user = new User( lastName, firstName, email, encryptedPassword, role, state, hasAccessToBO, creationDate );

                getEtherService().createUser( user );
            }
            else
                throw new WebException( WebException.WebCode.USER_ALREADY_EXISTS, "This email already corresponds to an User", WebException.getExceptionThrowable() );

        }
        catch( DigestException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }
        catch( NoSuchAlgorithmException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }

        return getAllAndWaitingUsers();
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject removeUser( @NotNull @ParamName(ParameterConstants.PARAMETER_ID) final Integer userId )
            throws WebException, ServiceException
    {
        getEtherService().removeUserById( userId );

        return getAllAndWaitingUsers();
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject modifyUser( @NotNull @ParamName(ParameterConstants.PARAMETER_ID) final Integer userId,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_NAME) final String lastName,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_FIRST_NAME) final String firstName,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_EMAIL) final String email,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_PWD) final String pwd,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_ROLE) final String role,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_STATE) final String state,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_HAS_ACCESS) final Boolean hasAccessToBO,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_KEEP_SAME_PASSWORD) final Boolean keepSamePassword )
            throws WebException, ServiceException
    {
        try
        {
            final User existingUser = getEtherService().getUserByEmail( email );
            if( null == existingUser || userId.equals( existingUser.getId() ) )
            {
                final User user = getEtherService().getUserById( userId );
                user.setLastName( lastName );
                user.setFirstName( firstName );
                user.setEmail( email );
                if( !keepSamePassword && null != pwd )
                {
                    final String encryptedPassword = EtherHelper.encryptPassword( pwd );
                    user.setPassword( encryptedPassword );
                }
                user.setRole( UserRole.valueOf( role ) );
                user.setState( UserState.valueOf( state ) );
                user.setAccessToBO( hasAccessToBO );

                getEtherService().updateUser( user );
            }
            else
                throw new WebException( WebException.WebCode.USER_ALREADY_EXISTS, "This email already corresponds to an User", WebException.getExceptionThrowable() );
        }
        catch( DigestException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }
        catch( NoSuchAlgorithmException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }

        return getAllAndWaitingUsers();
    }

    /**
     * This method changes the state user and sends an email to inform of the administrator decision.
     * If the email can't be sent, the state will be not changed
     *
     * @param userId
     * @param isAccepted
     * @return
     * @throws WebException
     * @throws ServiceException
     */
    @ControllerMethod(jsonResult = true)
    public JSONObject acceptOrRefuseUser( @NotNull @ParamName(ParameterConstants.PARAMETER_ID) final Integer userId,
                                          @ParamName(ParameterConstants.PARAMETER_OK) final boolean isAccepted )
            throws WebException, ServiceException
    {
        final User user = getEtherService().getUserById( userId );
        try
        {
            sendEmailToUser( user, isAccepted );
            getEtherService().acceptOrRefuseUser( userId, isAccepted );
        }
        catch( MessagingException e )
        {
            throw new WebException( WebException.WebCode.ERROR_EMAIL_CANNOT_BE_SEND, "The email cannot be send to the user : " + user.getEmail(), WebException.getExceptionThrowable() );
        }

        return getAllAndWaitingUsers();
    }

    /**
     * This method create and send an email to the user to inform of the administrator decision
     *
     * @param user
     * @param isAccepted
     * @throws WebException
     */
    private void sendEmailToUser( @NotNull final User user, final boolean isAccepted )
            throws MessagingException
    {
        final MailFactory mailFactory = (MailFactory) getServletContext().getAttribute( "APP_MAILFACTORY" );
        final String from = (String) getServletContext().getAttribute( "APP_WEBMASTER" );
        final String subject = "[MEGAPOLI] User inscription";
        String content = "Dear user, \n\nThe Megapoli administrator has ";
        if( isAccepted )
            content += "accepted your inscription. You can now access to data with you login (" + user.getEmail() + ") and the password you gave during inscription.\n";
        else
            content += "refused your inscription. You can reply to this email if you want more information.\n";

        content += "\nBest regards,\nMegapoli administrator";

        final Mail mail = new Mail( from, user.getEmail(), null, content, subject );
        mailFactory.sendMail( mail );
    }


    private JSONObject getAllAndWaitingUsers()
            throws ServiceException
    {
        final List<User> waitingUsers = getEtherService().getUsersByState( UserState.WAITING );
        final List<User> users = getEtherService().getAllUsersByNameOrder();

        final JSONObject result = new JSONObject();
        result.put( "jSonWaitingUsers", getJsonHelper().toJSON( waitingUsers ) );
        result.put( "jSonUsers", getJsonHelper().toJSON( users ) );
        return result;
    }

    private List<JSONObject> getJSONUserStates()
    {
        final UserState[] userStates = UserState.values();

        final List<JSONObject> jsonUserStates = new ArrayList<JSONObject>( userStates.length );

        for( final UserState userState : userStates )
        {
            final JSONObject jsonUserState = new JSONObject();
            jsonUserState.put( "text", userState.name() );
            jsonUserState.put( "value", userState.name() );
            jsonUserStates.add( jsonUserState );
        }
        return jsonUserStates;
    }

    private List<JSONObject> getJSONUserRoles()
    {
        final UserRole[] userRoles = UserRole.values();

        final List<JSONObject> jsonUserRoles = new ArrayList<JSONObject>( userRoles.length );

        for( final UserRole userRole : userRoles )
        {
            final JSONObject jsonUserRole = new JSONObject();
            jsonUserRole.put( "text", userRole.name() );
            jsonUserRole.put( "value", userRole.name() );
            jsonUserRoles.add( jsonUserRole );
        }
        return jsonUserRoles;
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerBackoffice.class );

    private static final String VIEW_BO_INDEX = "backoffice/index";
    private static final String VIEW_BO_USER = "backoffice/user";
}
