package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.Mandatory;
import com.ether.annotation.ParamName;
import com.medias.annuaire.Personne;
import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import com.medias.megapoli.struts.forms.DatasForm;
import com.medias.megapoli.trade.MesureAdapter;
import com.medias.megapoli.trade.ValeurAdapter;
import com.medias.megapoli.utils.Requete;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Document;
import org.jdom.output.XMLOutputter;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author vmipsl
 * @date 17 feb 2011
 */
public class ControllerVisualization
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    // Default view if methodName is unknown
    public ModelAndView home( final HttpServletRequest request, final HttpServletResponse response )
            throws EtherException
    {
        return new ModelAndView( "index" );
    }

    @ControllerMethod(view = VIEW_WORK)
    public Map<String, Object> viewInWork()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_VISUALIZATION)
    public Map<String, Object> view()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_VISUALIZATION_PARAMETER_BY_PLATEFORM)
    public Map<String, Object> viewParametersByPlateform()
            throws ServiceException
    {
        final List<Plateforme> plateforms = getEtherService().getAllPlateforms();
        final Date firstDate = getEtherService().getFirstDate();
        final Date endDate = getEtherService().getLastDate();

        final Map<String, Object> model = new HashMap<String, Object>();
        model.put( "plateforms", getJsonHelper().toJSON( plateforms ) );
        model.put( "axeTypesForFixedPlateforms", getJSONAxeTypesForFixedPlateforms() );
        model.put( "axeTypesForMobilePlateforms", getJSONAxeTypesForMobilePlateforms() );
        model.put( "firstDate", DateHelper.formatDate( firstDate, DateHelper.ENGLISH_DATE_PATTERN ) );
        model.put( "lastDate", DateHelper.formatDate( endDate, DateHelper.ENGLISH_DATE_PATTERN ) );
        return model;
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(jsonResult = true)
    public JSONObject searchParametersByPlateform( @Mandatory @ParamName(ParameterConstants.PARAMETER_ID) final Integer plateformId )
            throws ServiceException, EtherException
    {
        final List<Parametre> fullParametersByPlateform = getEtherService().getParametersByPlateformId( plateformId );
        final List<List<Parametre>> parametersByPlateform = manageMenusForParameterList( fullParametersByPlateform );

        final JSONObject result = new JSONObject();
        result.put( ParameterConstants.PARAMETER_PARAMETERS, getJsonHelper().toJSON( parametersByPlateform ) );
        return result;
    }

    /**
     * This method manage the sub-menus for the list of parameters (to avoid multiple "Particle Concentration" by example)
     * The fullParameters must be ordered with parameterName ascendant
     *
     * @param fullParameters
     * @return
     */
    @NotNull
    private List<List<Parametre>> manageMenusForParameterList( @NotNull final List<Parametre> fullParameters )
    {
        final List<List<Parametre>> parameterListWithMenu = new ArrayList<List<Parametre>>();

        int i = 0;
        while( i < fullParameters.size() )
        {
            final Parametre fullParameter = fullParameters.get( i );
            final int firstIndex = fullParameters.indexOf( fullParameter );
            final int lastIndex = fullParameters.lastIndexOf( fullParameter );
            final List<Parametre> parameters = fullParameters.subList( firstIndex, lastIndex + 1 );
            parameterListWithMenu.add( parameters );
            i += parameters.size();
        }

        return parameterListWithMenu;
    }


    @ControllerMethod(jsonResult = true, loginMandatory = true)
    public JSONObject downloadData( @ParamName(ParameterConstants.PARAMETER_DATE_BEGIN) final String dateBegin,
                                    @ParamName(ParameterConstants.PARAMETER_DATE_END) final String dateEnd,
                                    @ParamName(ParameterConstants.PARAMETER_PFID_PID_LIST) final String pIdPIdArrayString,
                                    @NotNull final Personne personne )
            throws ServiceException, WebException
    {
        final JSONObject result = new JSONObject();

        final List<Pair<String, String>> pfIdPIdList = EtherHelper.extractpfIdPIdListFromString( pIdPIdArrayString, String.class, String.class );
        if( null == pfIdPIdList )
            throw new WebException( WebException.WebCode.PLATEFORM_OR_PARAMETER_IS_NULL );

        final DatasForm dataForm = createDataForm( dateBegin, dateEnd, pfIdPIdList );

        final XMLOutputter outXml = new XMLOutputter();
        final Document docXml = Requete.toXml( dataForm, personne );
        final String requete = outXml.outputString( docXml );
        try
        {
            final String encoded = "requete=" + URLEncoder.encode( requete, "UTF-8" );
            // Configuration de l'URL
            final String urlCGIStr = (String) this.getServletContext().getAttribute( "APP_CGI" );
            final URL urlCGI = new URL( urlCGIStr );
            final URLConnection conn = urlCGI.openConnection();
            conn.setDoOutput( true );
            conn.setUseCaches( false );
            conn.setRequestProperty( "content-type", "application/x-www-form-urlencoded" );

            // Envoi de la requête
            final DataOutputStream out = new DataOutputStream( conn.getOutputStream() );
            out.writeBytes( encoded );
            out.flush();
            out.close();

            // Réponse du CGI
            final BufferedReader in = new BufferedReader( new InputStreamReader( conn.getInputStream() ) );
            String aLine;
            while( ( aLine = in.readLine() ) != null )
            {
                if( aLine.length() == 0 ) continue;
                if( aLine.startsWith( "1" ) )
                {
                    System.err.println( "ERROR :" + aLine );
                }
                else System.out.println( "REPONSE :" + aLine );
            }
        }
        catch( Exception e )
        {
            result.put( ParameterConstants.PARAMETER_RESULT, ParameterConstants.PARAMETER_NOT_OK );
            return result;
        }

        result.put( ParameterConstants.PARAMETER_RESULT, ParameterConstants.PARAMETER_OK );
        return result;
    }

    /**
     * This method create a dataForms to simulate and use the struts actions "DataAccessAction" and "DataSubmitAction"
     *
     * @param dateBegin
     * @param dateEnd
     * @param pfIdPIdList
     * @return
     * @throws WebException
     */
    @NotNull
    private DatasForm createDataForm( final String dateBegin, final String dateEnd, final List<Pair<String, String>> pfIdPIdList )
            throws WebException
    {
        final DatasForm dataForm = new DatasForm();

        // Plateforms and parameters
        final List<String> plateformIds = EtherHelper.getFirstValues( pfIdPIdList );
        final List<String> parameterIds = EtherHelper.getSecondValues( pfIdPIdList );
        dataForm.setSelectPlats( plateformIds.toArray( new String[plateformIds.size()] ) );
        dataForm.setSelectParams( parameterIds.toArray( new String[parameterIds.size()] ) );

        // Localisations
        final MesureAdapter mAdapter = new MesureAdapter();
        mAdapter.loadLocsFromDatabase( dataForm );

        // Dates
        final Calendar calendar = Calendar.getInstance();
        String formatedDateBegin = null;
        String formatedDateEnd = null;
        if( null != dateBegin && null != dateEnd && !"false".equals( dateBegin ) && !"false".equals( dateEnd ) )
        {
            try
            {
                calendar.setTimeInMillis( Long.valueOf( dateBegin ) );
                formatedDateBegin = DateHelper.formatDate( calendar.getTime(), DateHelper.FRENCH_DATE_PATTERN );
                calendar.setTimeInMillis( Long.valueOf( dateEnd ) );
                formatedDateEnd = DateHelper.formatDate( calendar.getTime(), DateHelper.FRENCH_DATE_PATTERN );
                dataForm.setDateDeb( formatedDateBegin );
                dataForm.setDateFin( formatedDateEnd );
            }
            catch( Exception e )
            {
                throw new WebException( WebException.WebCode.INVALID_DATE, e );
            }
        }
        else
            mAdapter.loadDatesFromDatabase( dataForm );

        // Fill dataForm
        final ValeurAdapter vAdapter = new ValeurAdapter();
        vAdapter.loadCountFromDatabase( dataForm );
        dataForm.setListeCoords( dataForm.getLatMin() + "," + dataForm.getLatMax() + "," + dataForm.getLonMin() + "," + dataForm.getLonMax() );
        dataForm.setListeDates( dataForm.getDateDeb() + "," + dataForm.getDateFin() );
        dataForm.setOutput( FORMAT_DEFAULT );
        dataForm.setCompression( COMPRESSION_DEFAULT );

        return dataForm;
    }

    private List<JSONObject> getJSONAxeTypesForFixedPlateforms()
    {
        final AxeTypeForFixedPlateform[] axeTypes = AxeTypeForFixedPlateform.values();

        final List<JSONObject> jsonAxeTypes = new ArrayList<JSONObject>( axeTypes.length );

        for( final AxeTypeForFixedPlateform axeType : axeTypes )
        {
            final JSONObject jsonAxeType = new JSONObject();
            jsonAxeType.put( "text", axeType.name() );
            jsonAxeType.put( "value", axeType.name() );
            jsonAxeTypes.add( jsonAxeType );
        }
        return jsonAxeTypes;
    }

    private List<JSONObject> getJSONAxeTypesForMobilePlateforms()
    {
        final AxeTypeForMobilePlateform[] axeTypes = AxeTypeForMobilePlateform.values();

        final List<JSONObject> jsonAxeTypes = new ArrayList<JSONObject>( axeTypes.length );

        for( final AxeTypeForMobilePlateform axeType : axeTypes )
        {
            final JSONObject jsonAxeType = new JSONObject();
            jsonAxeType.put( "text", axeType.name() );
            jsonAxeType.put( "value", axeType.name() );
            jsonAxeTypes.add( jsonAxeType );
        }
        return jsonAxeTypes;
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerVisualization.class );

    private static final String VIEW_WORK = "visualization/inWork";
    private static final String VIEW_VISUALIZATION = "visualization/visu";
    private static final String VIEW_VISUALIZATION_PARAMETER_BY_PLATEFORM = "visualization/visu_parameter_by_pf";
    private static final String VIEW_DOWNLOAD_OK = "data/access/extract3";

    private static final String FORMAT_DEFAULT = "NASA-AMES";
    private static final String COMPRESSION_DEFAULT = "None";
 }